@@ -40,9 +40,6 @@
 #include "f2c.h"
 #include <float.h>
 
-/*Defined in resample.c*/
-void MyTransForm( TrformStr *TrPtr, fDesc *fD, int color, int imageNum);
-
 #define C_FACTOR	100.0
 
 static  AlignInfo	*g;	// This struct holds all informations for the optimization
@@ -170,8 +167,8 @@ void adjust(TrformStr *TrPtr, aPrefs *prefs)
 							}
 						}
 						if(ainf.numPts > 0) free(ainf.cpt);
+						free( script );
 					}
-					free( script );
 				}
 #endif
 			}else{
@@ -443,75 +440,23 @@ void adjust(TrformStr *TrPtr, aPrefs *prefs)
 }
 
 
-
-
 // Make a pano in TrPtr->dest (must be allocated and all set!)
 // using parameters in aPrefs (ignore image parameters in TrPtr !)
 
 void MakePano( TrformStr *TrPtr, aPrefs *aP )
 {
-	struct 	MakeParams	mp;
-	fDesc 	stack[15], fD;		// Parameters for execute 
-	void	*morph[3];	
-
-	int 	i,k, kstart, kend, color;
-
-	TrPtr->success = 1;
-	
-	if( CheckMakeParams( aP) != 0)
-	{
-		TrPtr->success = 0;
-		return;
-	}
-
-
-	if(  isColorSpecific( &(aP->im.cP) ) )			// Color dependent
-	{
-		kstart 	= 1; kend	= 4;
-	}
-	else 											// Color independent
-	{
-		kstart	= 0; kend	= 1;
-	}
-				
-	for( k = kstart; k < kend; k++ )
-	{
-		color = k-1; if( color < 0 ) color = 0;
-		SetMakeParams( stack, &mp, &(aP->im) , &(aP->pano), color );
-		
-		if( aP->nt > 0 )	// Morphing requested
-		{
-			morph[0] = (void*)aP->td;
-			morph[1] = (void*)aP->ts;
-			morph[2] = (void*)&aP->nt;
-
-			i=0; while( stack[i].func != NULL && i<14 ) i++;
-			if( i!=14 )
-			{
-				for(i=14; i>0; i--)
-				{
-					memcpy( &stack[i], &stack[i-1], sizeof( fDesc ));
-				}
-				stack[0].func 		= tmorph;
-				stack[0].param 		= (void*)morph;
-			}
-		}
-					
-			
-		
-		if( TrPtr->success != 0)
-		{
-			fD.func = execute_stack; fD.param = stack;
-			transForm( TrPtr,  &fD , k);
-		}
-	}
+	MyMakePano( TrPtr, aP, 1 );
 }
 
-/*This function was added by Kekus Digital on 18/9/2002. This function takes the parameter 'imageNum' which repesents the index of the image that has to be converted.*/
+
+/*This function was added by Kekus Digital on 18/9/2002. 
+This function takes the parameter 'imageNum' which repesents the index 
+of the image that has to be converted.*/
 void MyMakePano( TrformStr *TrPtr, aPrefs *aP, int imageNum )
 {
-	struct 	MakeParams	mp;
+	struct 	MakeParams	mp,mpinv;
 	fDesc 	stack[15], fD;		// Parameters for execute 
+	fDesc 	invstack[15], finvD;		// Invers Parameters for execute 
 	void	*morph[3];	
 
 	int 	i,k, kstart, kend, color;
@@ -538,6 +483,7 @@ void MyMakePano( TrformStr *TrPtr, aPrefs *aP, int imageNum )
 	{
 		color = k-1; if( color < 0 ) color = 0;
 		SetMakeParams( stack, &mp, &(aP->im) , &(aP->pano), color );
+		SetInvMakeParamsCorrect( invstack, &mpinv, &(aP->im) , &(aP->pano), color );
 		
 		if( aP->nt > 0 )	// Morphing requested
 		{
@@ -556,26 +502,26 @@ void MyMakePano( TrformStr *TrPtr, aPrefs *aP, int imageNum )
 				stack[0].param 		= (void*)morph;
 			}
 		}
-					
-			
-		
+
 		if( TrPtr->success != 0)
 		{
 			fD.func = execute_stack; fD.param = stack;
-			MyTransForm( TrPtr,  &fD , k, imageNum);
+			finvD.func = execute_stack; finvD.param = invstack;
+
+			transFormEx( TrPtr,  &fD , &finvD , k, imageNum );
 		}
 	}
 }
 
-
 // Extract image from pano in TrPtr->src 
 // using parameters in prefs (ignore image parameters
 // in TrPtr)
 
 void ExtractStill( TrformStr *TrPtr , aPrefs *aP )
 {
-	struct 	MakeParams	mp;
+	struct 	MakeParams	mp,mpinv;
 	fDesc 	stack[15], fD;		// Parameters for execute 
+	fDesc 	stackinv[15], fDinv;		// Invers Parameters for execute 
 
 	int 	k, kstart, kend, color;
 
@@ -586,7 +532,6 @@ void ExtractStill( TrformStr *TrPtr , aPrefs *aP )
 		TrPtr->success = 0;
 		return;
 	}
-
 		
 
 	if( isColorSpecific( &(aP->im.cP) ) )			// Color dependent
@@ -601,12 +546,14 @@ void ExtractStill( TrformStr *TrPtr , aPrefs *aP )
 	for( k = kstart; k < kend; k++ )
 	{
 		color = k-1; if( color < 0 ) color = 0;
-		SetInvMakeParams( stack, &mp,  &(aP->im) , &(aP->pano), color );
+		SetInvMakeParamsCorrect( stack, &mp, &(aP->im), &(aP->pano), color );
+		SetMakeParams( stackinv, &mpinv, &(aP->im), &(aP->pano), color );
 		
 		if( TrPtr->success != 0)
 		{
 			fD.func = execute_stack; fD.param = stack;
-			transForm( TrPtr,  &fD , k);
+			fDinv.func = execute_stack; fDinv.param = stackinv;
+			transFormEx( TrPtr, &fD, &fDinv, k, 1 );
 		}
 	}
 }
@@ -655,10 +602,10 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
         {
           if(im->cP.cutFrame)
           {
-            image_selection_width=im->selection.right-im->selection.left;
-            image_selection_height=im->selection.bottom-im->selection.top;
-            mp->horizontal += (im->selection.right+im->selection.left-im->width)/2.0;
-            mp->vertical += (im->selection.bottom+im->selection.top-im->height)/2.0;
+            image_selection_width  = im->selection.right  - im->selection.left;
+            image_selection_height = im->selection.bottom - im->selection.top;
+            mp->horizontal += (im->selection.right  + im->selection.left - im->width)/2.0;
+            mp->vertical   += (im->selection.bottom + im->selection.top  - im->height)/2.0;
           }
         }
 
@@ -708,6 +655,7 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 	mp->shear[1] 	= im->cP.shear_y / image_selection_width;
 	mp->rot[0]		= mp->distance * PI;								// 180 in screenpoints
 	mp->rot[1]		= -im->yaw *  mp->distance * PI / 180.0; 			//    rotation angle in screenpoints
+
 	mp->perspect[0] = (void*)(mp->mt);
 	mp->perspect[1] = (void*)&(mp->distance);
 			
@@ -975,11 +923,50 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 	stack[i].func = (trfn)NULL;
 }
 
+void 	SetInvMakeParamsCorrect( struct fDesc *stack, struct MakeParams *mp, Image *im , Image *pn, int color )
+{
+/* Thomas Rauscher, Sep 2005: Transfered the changes of Joost Nieuwenhuijse for MakeParams
+   to the inverse function. This has broken the optimizer, now there are two functions.
+*/
 
+	Image imSel;	/* create a tempory copy of the image to manipulate */
+	memcpy( &imSel, im, sizeof(Image));
 
+	if(im->cP.horizontal)
+    {
+        mp->horizontal = im->cP.horizontal_params[color];
+    }
+    else
+    {
+        mp->horizontal = 0;
+    }
 
-	
+	if(im->cP.vertical)
+    {
+        mp->vertical = im->cP.vertical_params[color];
+    }
+    else
+    {
+        mp->vertical = 0;
+    }
 
+	if( (im->selection.left != 0) || (im->selection.top != 0) || (im->selection.bottom != 0) || (im->selection.right != 0) )
+    {
+        if(im->cP.cutFrame)
+        {
+			imSel.width = im->selection.right  - im->selection.left;
+			imSel.width = im->selection.bottom - im->selection.top;
+
+			mp->horizontal += (im->selection.right  + im->selection.left - im->width)/2.0;
+			mp->vertical   += (im->selection.bottom + im->selection.top  - im->height)/2.0;
+
+			imSel.cP.horizontal_params[color] = mp->horizontal;
+			imSel.cP.vertical_params[color]   = mp->vertical;
+        }
+    }
+
+	SetInvMakeParams( stack, mp, &imSel, pn, color );
+}
 	
 
 
