@@ -75,14 +75,17 @@ enum{
 
 
 
-#define	DBL_TO_UC( c, x )	if(x>255.0) c=255U;								\
-								else if(x<0.0) c=0;							\
-								else c=(unsigned char)floor(x+0.5);
+#define	DBL_TO_UC( c, x )	if((x)>255.0) c=255U;								\
+								else if((x)<0.0) c=0;							\
+								else c=(unsigned char)floor((x)+0.5);
 
-#define	DBL_TO_US( c, x )	if(x>65535.0) c=65535U;							\
-								else if(x<0.0) c=0;							\
-								else c=(unsigned short)floor(x+0.5);
+#define	DBL_TO_US( c, x )	if((x)>65535.0) c=65535U;							\
+								else if((x)<0.0) c=0;							\
+								else c=(unsigned short)floor((x)+0.5);
 
+#define	DBL_TO_FL( c, x )	if((x)>1e+038) c=1e+038;							\
+								else if((x)<0.0) c=0;							\
+								else c=(float)(x);
 
 
 #define MAX_FISHEYE_FOV		160.0
@@ -442,7 +445,7 @@ typedef struct MultiLayerImage MultiLayerImage;
 
 
 	
-	
+void filter_main( TrformStr *TrPtr, sPrefs *spref);	
 
 
 // Transformation function type (we have only one...)
@@ -478,9 +481,9 @@ typedef		void (*intFunc)( unsigned char *dst, 	unsigned char **rgb,
 							register double Dy,
 							int color, int SamplesPerPixel);
 
+// Filter function type for anti aliasing Filter
 
-
-
+typedef double  (*aaFilter)(const double,const double);
 
 // Gamma Correction
 
@@ -648,11 +651,13 @@ int		ReadMorphPoints( char *script, AlignInfo *gl, int nIm );
 
 void 	addAlpha			( Image *im ); 
 void 	transForm			( TrformStr *TrPtr, fDesc *fD, int color);
+void 	transFormEx			( TrformStr *TrPtr, fDesc *fD, fDesc *finvD, int color, int imageNum);
 void    filter				( TrformStr *TrPtr, flfn func, flfn16 func16, void* params, int color);		
 void 	CopyImageData		( Image *dest, Image *src );
 void 	laplace				( Image *im );
 void 	blurr				( Image *im );
 void 	MakePano			( TrformStr *TrPtr, aPrefs *aP);
+void	MyMakePano			( TrformStr *TrPtr, aPrefs *aP, int imageNum );
 void 	ExtractStill		( TrformStr *TrPtr , aPrefs *p );
 int 	HaveEqualSize		( Image *im1, Image *im2 );
 int 	merge				( Image *dst, Image *src, int feather, int showprogress, int seam );
@@ -675,6 +680,9 @@ void 	OneToTwoByte		( Image *im );
 void 	TwoToOneByte		( Image *im );
 void 	SetMakeParams		( struct fDesc *stack, struct MakeParams *mp, Image *im , Image *pn, int color );
 void 	SetInvMakeParams	( struct fDesc *stack, struct MakeParams *mp, Image *im , Image *pn, int color );
+// same as SetInvMakeParams but includes Joosts inverted changes to SetMakeParams
+void 	SetInvMakeParamsCorrect( struct fDesc *stack, struct MakeParams *mp, Image *im , Image *pn, int color );
+
 void 	GetControlPointCoordinates(int i, double *x, double *y, AlignInfo *gl );
 void 	ARGBtoRGBA(UCHAR* buf, int width, int bitsPerPixel);
 void 	RGBAtoARGB(UCHAR* buf, int width, int bitsPerPixel);
